package com.rf.kjb.chat.service.impl;

import com.rf.kjb.chat.dao.domain.ChatRecordEntity;
import com.rf.kjb.chat.dao.repository.ChatRecordRepository;
import com.rf.kjb.chat.service.ChatRecordService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @Author:zzf
 * @Date:2023/9/14:15:27
 * @Description:
 */
@Service
public class ChatRecordServiceImpl implements ChatRecordService {
    @Autowired
    private ChatRecordRepository repository;
    @Override
    public void save(ChatRecordEntity chatRecordEntity) {
        repository.save(chatRecordEntity);
    }

    @Override
    public Page<ChatRecordEntity> find(String identifier, String beginDate,String endDate, int pageNum, int pageSize) {
        Specification<ChatRecordEntity> specification = (root, query, cb) -> {
            List<javax.persistence.criteria.Predicate> predicateList = new ArrayList<>();
            if(StringUtils.isNotBlank(identifier)){
                predicateList.add(cb.or(cb.like(root.get("identifier"),"%"+identifier+"%"),cb.like(root.get("userName"),"%"+identifier+"%")));
            }
            try {
                if(StringUtils.isNotBlank(beginDate)){
                        predicateList.add(cb.greaterThanOrEqualTo(root.get("createTime"),new SimpleDateFormat("yyyy-MM-dd").parse(beginDate)));
                }

                if(StringUtils.isNotBlank(endDate)){
                    predicateList.add(cb.lessThanOrEqualTo(root.get("createTime"),new SimpleDateFormat("yyyy-MM-dd").parse(endDate)));
                }
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.asc(root.get("createTime")));
            return query.getRestriction();
        };
        if (pageNum <= 0) {
            pageNum = 1;
        }
        if(pageSize <= 0){
            pageSize = 10;
        }
        return this.repository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }

    @Override
    public ChatRecordEntity findById(String id) {
        return repository.findById(id).get();
    }

    @Override
    public List<ChatRecordEntity> findAll(String identifier, String beginDate, String endDate) {
        Specification<ChatRecordEntity> specification = (root, query, cb) -> {
            List<javax.persistence.criteria.Predicate> predicateList = new ArrayList<>();
            if(StringUtils.isNotBlank(identifier)){
                predicateList.add(cb.or(cb.like(root.get("identifier"),"%"+identifier+"%"),cb.like(root.get("userName"),"%"+identifier+"%")));
            }
            try {
                if(StringUtils.isNotBlank(beginDate)){
                    predicateList.add(cb.greaterThanOrEqualTo(root.get("createTime"),new SimpleDateFormat("yyyy-MM-dd").parse(beginDate)));
                }

                if(StringUtils.isNotBlank(endDate)){
                    predicateList.add(cb.lessThanOrEqualTo(root.get("createTime"),new SimpleDateFormat("yyyy-MM-dd").parse(endDate)));
                }
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.asc(root.get("createTime")));
            return query.getRestriction();
        };
        return this.repository.findAll(specification);
    }
}
