package com.rf.kjb.filter;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * @author lpf
 * {@code @description:}
 * {@code @date} 2021/12/2822:06
 */
@Configuration
public class JWTInterceptorConfig implements WebMvcConfigurer {

    @Autowired
    private UserCheckInterceptor userCheckInterceptor;

    @Autowired
    private JWTInterceptor jwtInterceptor;


    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        //定义排除swagger访问的路径配置
        String[] swaggerExcludes = new String[]{"/swagger*/**", "/v3/api-docs", "/webjars/**","/doc.html"};
        String[] user = new String[]{"/v1/user/login", "/v1/user/register","/v1/user/**/checkUser","/v1/user/hello"};
        String[] record = new String[]{"/v1/record/download/**","/v1/recordPhy/download","/v1/recordPhy/export/all/pulse/target"};
        String[] faqImport = new String[]{"/chat/import/**/faq"};
        registry.addInterceptor(jwtInterceptor)
                .addPathPatterns("/**")
                .excludePathPatterns("/", "/index**", "/error")
                .excludePathPatterns(user)
                .excludePathPatterns(swaggerExcludes)
                .excludePathPatterns(record)
                .excludePathPatterns(faqImport)
                .excludePathPatterns("/favicon.ico")
                .excludePathPatterns("/druid/**")
                .excludePathPatterns("/static/**");//排除静态资源
        registry.addInterceptor(userCheckInterceptor)
                .addPathPatterns("/**")
                .excludePathPatterns("/", "/index.html", "/error")
                .excludePathPatterns(user)
                .excludePathPatterns(swaggerExcludes)
                .excludePathPatterns(record)
                .excludePathPatterns(faqImport)
                .excludePathPatterns("/favicon.ico")
                .excludePathPatterns("/druid/**")
                .excludePathPatterns("/static/**");//排除静态资源

    }

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        registry.addResourceHandler("/static/**")
                .addResourceLocations("classpath:/static/");
    }
}
