package com.rf.kjb.hospital.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.hospital.dao.model.HospitalEntity;
import com.rf.kjb.hospital.dao.repository.HospitalRepository;
import com.rf.kjb.hospital.service.HospitalService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author:zzf
 * @Date:2023/5/12:11:28
 * @Description:
 */
@Service
public class HospitalServiceImpl implements HospitalService {

    @Autowired
    private HospitalRepository repository;
    @Override
    public HospitalEntity save(HospitalEntity hospitalEntity) {
        return this.repository.save(hospitalEntity);
    }

    @Override
    public Page<HospitalEntity> findAll(JSONObject jsonObject) {
        Specification<HospitalEntity> specification = (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if(jsonObject.containsKey("id") && StringUtils.isNotEmpty(jsonObject.getString("id"))){
                predicateList.add(criteriaBuilder.equal(root.get("id"),jsonObject.getString("id")));
            }
            //id:数据库主键；provinceCode：地区编码；hospitalName：名称；address:地址；pageNum：可为空，默认1 ；pageSize：可为空，默认10
            if(jsonObject.containsKey("provinceCode") && StringUtils.isNotEmpty(jsonObject.getString("provinceCode"))){
                predicateList.add(criteriaBuilder.equal(root.get("provinceCode"),jsonObject.getString("provinceCode")));
            }
            if(jsonObject.containsKey("hospitalName") && StringUtils.isNotEmpty(jsonObject.getString("hospitalName"))){
                predicateList.add(criteriaBuilder.like(root.get("hospitalName"),"%"+jsonObject.getString("hospitalName")+"%"));
            }
            if(jsonObject.containsKey("address") && StringUtils.isNotEmpty(jsonObject.getString("address"))){
                predicateList.add(criteriaBuilder.like(root.get("address"),"%"+jsonObject.getString("address")+"%"));
            }
            query.where(criteriaBuilder.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(criteriaBuilder.desc(root.get("createTime")),criteriaBuilder.desc(root.get("updateTime")));
            return query.getRestriction();
        };
        int pageNum = (jsonObject.containsKey("pageNum") && jsonObject.getInteger("pageNum") >0)?jsonObject.getIntValue("pageNum"): 1;
        int pageSize = (jsonObject.containsKey("pageSize") && jsonObject.getInteger("pageSize") >0)?jsonObject.getIntValue("pageSize"): 10;
        return this.repository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }
}
