package com.rf.kjb.scale.dao.repository;

import com.rf.kjb.base.repository.BaseRepository;
import com.rf.kjb.scale.dao.model.SubjectEntity;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;
import java.util.List;

/**
 * @author zzf
 * @description:
 * @date 2021/5/17 9:46
 */
public interface SubjectRepository extends BaseRepository<SubjectEntity, String> {
    /**
     * 查询
     *
     * @param type 0：量表   1：认知任务
     * @return
     */
    @Query(value = "select * from t_subject_info where type = :type", nativeQuery = true)
    List<SubjectEntity> findAll(@Param("type") String type);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "delete from t_subject_info", nativeQuery = true)
    void deleteAll();

    /**
     * 检索，根据中文名、英文名或录入人
     *
     * @param pageNum
     * @param pageSize
     * @param searchName
     * @return
     */
    @Query(value = "select * from t_subject_info tsi where name like %:searchName% or ename like %:searchName%  order by flag desc  limit :pageNum ,:pageSize ", nativeQuery = true)
    List<SubjectEntity> findByNameOrUploader(@Param("pageNum") int pageNum, @Param("pageSize") int pageSize, @Param("searchName") String searchName);

    /**
     * 查询  分页
     *
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Query(value = "select * from t_subject_info order by status,flag desc  limit :pageNum ,:pageSize ", nativeQuery = true)
    List<SubjectEntity> findAll(@Param("pageNum") int pageNum, @Param("pageSize") int pageSize);

    /**
     * 根据flag查询
     *
     * @return
     */
    @Query(value = "select * from t_subject_info where flag = :flag ", nativeQuery = true)
    SubjectEntity getSubjectByFlag(@Param("flag") String flag);

    /**
     * 根据flag记录测试数
     *
     * @return
     */
    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "UPDATE t_subject_info SET test_num = test_num+1 WHERE flag = :flag ", nativeQuery = true)
    void addTestNum(@Param("flag") String flag);

    /**
     * 修改量表测试结果效验状态
     *
     * @return
     */
    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "UPDATE t_subject_info SET status = :status WHERE flag = :flag ", nativeQuery = true)
    void updateScaleStatus(@Param("flag") String flag, @Param("status") String status);

    /**
     * 根据id修改指定字段
     *
     * @return
     */
    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "UPDATE t_subject_info SET description = :description WHERE id = :id ", nativeQuery = true)
    void updateSubject(@Param("description") String description, @Param("id") String id);


    @Query(value = "select * from t_subject_info where name like :scaleName",nativeQuery = true)
    List<SubjectEntity> findByName(@Param("scaleName") String scaleName);
}
