package com.rf.kjb.scale.rest;

import com.rf.kjb.base.rest.BaseController;
import com.rf.kjb.exception.ErrorCode;
import com.rf.kjb.scale.dao.model.CategoryEntry;
import com.rf.kjb.scale.service.CategoryService;
import com.rf.kjb.utils.Result;
import com.rf.kjb.utils.Utils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author zsy
 * @description:量表类别信息管理
 * @date 2021/6/17 15:55
 */

@Slf4j
@RestController
@RequestMapping("/v1/category")
@Api(tags = "量表类别信息管理")
public class CategoryController extends BaseController {

    @Autowired
    private CategoryService categoryService;

    /**
     * 添加量表类别信息
     *
     * @param categoryEntity
     * @return
     */
    @PostMapping("/save")
    @ApiOperation("添加量表类别信息")
    public Result saveCategoryInfo(@RequestBody CategoryEntry categoryEntity) {

        assert StringUtils.isNotBlank(categoryEntity.getName()) : "类别名称为空";
        assert StringUtils.isNotBlank(categoryEntity.getEname()) : "类别英文名为空";
        assert StringUtils.isNotBlank(categoryEntity.getSequence()) : "序号为空";
        List<CategoryEntry> categoryEntryList = this.categoryService.getCategoryByName(categoryEntity);
        if(Utils.isNotBlank(categoryEntryList)){
            return fail(ErrorCode.CATEGORY_EXIST);
        }
        if(categoryEntity.getCreateTime() == null){
            categoryEntity.setCreateTime(new Date());
        }else{
            categoryEntity.setUpdateTime(new Date());
        }

        this.categoryService.save(categoryEntity);
        return success();
    }

    /**
     * 量表查询所有类别信息
     *
     * @return
     */
    @GetMapping("/getCategoryList")
    @ApiOperation(value = "量表查询所有类别信息")
    public Result getCategoryList() {
        List<CategoryEntry> categoryEntryList = this.categoryService.findAll();
        return success(categoryEntryList);
    }

}
