package com.rf.kjb.scale.rest;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.base.rest.BaseController;
import com.rf.kjb.scale.dao.dto.RecordEntityDto;
import com.rf.kjb.scale.dao.model.RecordEntity;
import com.rf.kjb.scale.dao.model.SubjectEntity;
import com.rf.kjb.scale.service.RecordService;
import com.rf.kjb.scale.service.SubjectService;
import com.rf.kjb.scale.util.DateUtil;
import com.rf.kjb.user.dao.model.UserEntry;
import com.rf.kjb.user.service.UserService;
import com.rf.kjb.utils.Constant;
import com.rf.kjb.utils.Result;
import com.rf.kjb.utils.Utils;
import com.rf.kjb.utils.ZipUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.stream.Stream;

/**
 * @Author:zzf
 * {@code @Date:2022/8/3:15:29}
 * {@code @Description:测试记录查询}
 */
@RestController
@RequestMapping("/v1/record")
@Slf4j
@Api(tags = "量表测试记录")
public class RecordController extends BaseController {

    @Autowired
    private RecordService recordService;

    @Autowired
    private UserService userService;

    @Autowired
    private SubjectService subjectService;

    /**
     * 查询做过量表的用户
     * @return
     */
    @PostMapping("/users")
    @ApiOperation(value = "量表维度查询用户",notes = "flag :量表flag  startTime ：开始时间 endTime：结束时间  userName：用户名 pageNum:页数  pageSize：每页记录数")
    public Result<JSONObject> userList(@RequestBody String json){
        List<String> identifiers;
        List<UserEntry> userEntries = new ArrayList<>();
        identifiers = this.recordService.findIdentifiers(json);
        int count = this.recordService.findIdentifiersCount(json);
        if(Utils.isNotBlank(identifiers)){
            userEntries = this.userService.findByIdentifiers(identifiers);
        }
        JSONObject resultJson = new JSONObject();
        resultJson.put("data",userEntries);
        resultJson.put("allNum",count);
        return  success(resultJson);
    }

    @PostMapping("/find")
    @ApiOperation(value = "查询",notes = "pageNum:当前页数 默认1  pageSize：每页记录数，默认10 startDate：测试起始时间，可为空 endDate：测试结束时间 可为空；identifier：用户编号 可为空 scaleName:量表名称模糊查询")
    public Result<JSONObject> find(@RequestBody String json){
        List<RecordEntityDto> dtos = new ArrayList<>();
        int allNum = 0;
        Page<RecordEntity> page = this.recordService.find(json);
        if(page.getTotalElements() >0){
            allNum = (int) page.getTotalElements();
            page.getContent().forEach(item ->{
                RecordEntityDto dto = copyEntry(item);
                SubjectEntity subjectEntity = this.subjectService.getSubjectByFlag(item.getFlag());
                dto.setDescription(subjectEntity.getDescription());
                dtos.add(dto);
            });
        }
        JSONObject resultJson = new JSONObject();
        resultJson.put("data",dtos);
        resultJson.put("allNum",allNum);
        return success(resultJson);
    }

    @PostMapping("/find/{id}")
    @ApiOperation(value = "详情查询",notes = "id 主键 id")
    public Result<JSONObject> findById(@PathVariable String id ){
        RecordEntity entry  = this.recordService.findById(id);
        assert entry != null : "测试记录不存在";
        UserEntry userInfo = this.userService.findByIdentifier(entry.getIdentifier());
        assert userInfo != null : "用户不存在";
        SubjectEntity subjectEntity = this.subjectService.getSubjectByFlag(entry.getFlag());

        JSONObject resultJson = new JSONObject();
        resultJson.put("subject",subjectEntity);
        resultJson.put("userInfo",userInfo);
        resultJson.put("userRecordEntity",entry);

        return success(resultJson);
    }

    @PostMapping("/download")
    @ApiOperation(value = "用户测试结果批量下载",notes = "ids :数组，下载的结果id，为空时表示全部下载；flag :量表flag")
    public String downloadResult(@RequestBody String json, HttpServletResponse response){
        JSONObject condition = JSONObject.parseObject(json);
        Workbook wb = new XSSFWorkbook();
        //查询测试记录 不分页
        List<RecordEntity> recordEntities = this.recordService.findNoPage(condition.toJSONString());
        if(!Utils.isNotBlank(recordEntities)){
            return  "无测试记录";
        }
        RecordEntity temp = recordEntities.get(0);
        JSONArray dbJsonList = JSONArray.parseArray(temp.getTestResult());
        JSONObject dbJson = dbJsonList.getJSONObject(0);
        JSONArray tableContext = dbJson.getJSONArray("tableContext");
        StringBuilder testResults = new StringBuilder();
        for (int i = 0; i < tableContext.size(); i++) {
            JSONObject jsonObject = tableContext.getJSONObject(i);
            testResults.append(jsonObject.getString("name")).append(":").append(jsonObject.getJSONArray("value").getString(0)).append(";");
        }
        System.out.println("testresults==="+testResults);
        String [] splits = testResults.substring(0,testResults.length()-1).split(";");
        System.out.println("splits.length=="+splits.length);
        for (int i = 0; i < splits.length; i++) {
            System.out.println(splits[i]);
        }
        //标题行抽出字段
        String [] title = new String[splits.length + 6];
        title[0] = "姓名";
        title[1] = "性别";
        title[2] = "编号";
        title[3] = "年龄";
        title[4] = "测试时间";
        title[5] = "测试名称";
        for (int i = 0; i < splits.length; i++) {
            title[i + 6] = splits[i].split(":")[0];
        }
        Sheet downSheet = wb.createSheet(Constant.BATCH_EXPORT_RESULT_SHEET_NAME);
        //获取表头行
        Row titleRow = downSheet.createRow(0);
        //创建单元格，设置style居中，字体，单元格大小等
        CellStyle style = wb.createCellStyle();
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HorizontalAlignment.CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        Cell cell;
        //把已经写好的标题行写入excel文件中
        for (int i = 0; i < title.length; i++) {
            cell = titleRow.createCell(i);
            cell.setCellValue(title[i]);
            cell.setCellStyle(style);
        }
        Row row;
        Map<String,String> subjectMap = loadAllSubject();
        for (int i = recordEntities.size()-1; i >= 0; i--) {
            RecordEntity recordEntity = recordEntities.get(i);

            row = downSheet.createRow(i+1);
            row.setRowStyle(style);
            //用户信息
            UserEntry userEntry = this.userService.findByIdentifier(recordEntity.getIdentifier());
            //姓名
            Cell cellUserName = row.createCell(0);
            cellUserName.setCellValue(userEntry.getUserName());
            //性别
            Cell cellGender = row.createCell(1);
            cellGender.setCellValue(userEntry.getGender() == 1?"男":"女");
            //编号
            Cell cellIdentifier = row.createCell(2);
            cellIdentifier.setCellValue(userEntry.getIdentifier());
            //年龄
            Cell cellAge = row.createCell(3);
            cellAge.setCellValue(userEntry.getAge());
            //测试时间
            Cell cellTestTime = row.createCell(4);
            cellTestTime.setCellValue(recordEntity.getTestDate());
            //测试名称
            Cell cellScaleName = row.createCell(5);
            cellScaleName.setCellValue(subjectMap.get(recordEntity.getFlag()));

            tableContext = JSONArray.parseArray(recordEntity.getTestResult()).getJSONObject(0).getJSONArray("tableContext");
            StringBuilder testResult = new StringBuilder();
            for (int y = 0; y < tableContext.size(); y++) {
                JSONObject jsonObject = tableContext.getJSONObject(y);
                testResult.append(jsonObject.getString("name")).append(":").append(jsonObject.getJSONArray("value").getString(0)).append(";");
            }

            String[] splitl = testResult.substring(0,testResult.length()-1).split(";");
            for (int z = 0; z < splitl.length; z++) {
                String[] a = splitl[z].split(":");
                Cell rowCell = row.createCell(z + 6);
                rowCell.setCellValue(a[1]);
                rowCell.setCellStyle(style);
            }
        }
        //设置单元个宽度自适应，在此基础上把宽度调至1.5倍
        for (int i = 0; i < title.length; i++) {
            downSheet.autoSizeColumn(i, true);
            int colWidth = downSheet.getColumnWidth(i) * 2;
            if (colWidth < 255 * 256) {
                downSheet.setColumnWidth(i, Math.max(colWidth, 3000));
            } else {
                downSheet.setColumnWidth(i, 6000);
            }
        }
        //获取配置文件中保存对应excel文件的路径
        String foldDir = "./心理检测/检测结果/" + DateUtil.getNowTime(DateUtil.DEFAULT_DATE_PATTERN) + "/" + Constant.BATCH_EXPORT_RESULT_SHEET_NAME;
        //创建上传文件目录
        File fileDir = new File(foldDir);
        if(!fileDir.exists())  fileDir.mkdirs();

        //设置文件名
        String fileName = this.subjectService.getSubjectByFlag(temp.getFlag()).getName()+ Constant.SPLIT_CHAR + DateUtil.getNowTime(DateUtil.CN_DATE) +".xlsx";
        OutputStream fileOut = null;
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        OutputStream os = null;
        try{
            fileOut = new FileOutputStream(foldDir+"/"+fileName);
            wb.write(fileOut);
            fileOut.close();
            File file = new File(foldDir+"/"+fileName);
            if(file.exists()){
                /*response.reset();
                response.setContentType("application/octet-stream");
                response.setCharacterEncoding("utf-8");
                response.setContentLength((int) file.length());
                response.setHeader("Content-Disposition", "attachment;filename=" + java.net.URLEncoder.encode(file.getName(), "UTF-8"));

                byte[] buffer = new byte[1024];
                fis = new FileInputStream(file);
                bis = new BufferedInputStream(fis);
                os = response.getOutputStream();
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
                return "下载成功";*/
                return com.rf.kjb.utils.FileUtils.downloadFile(file,"",fileName,response);
//                return downloadFile(file,"",fileName,response);
            }
        } catch (FileNotFoundException e) {
            log.error("未找到文件：");
            e.printStackTrace();
            return "下载失败";
        } catch (IOException e) {
            log.error("文件写入失败");
            e.printStackTrace();
            return "下载失败";
        }finally {
            try{
                if(fileOut != null) fileOut.close();

            } catch (IOException e) {
               log.error("关闭系统I/O资源失败");
            }
        }
        return "下载失败";
    }


    @PostMapping("/findByFlag")
    @ApiOperation(value = "根据flag查询量表测试记录，包含用户信息",notes = "flag:量表标志 pageNum：页数 pageSize：每页记录数 startDate:起始时间  endDate：结束时间")
    public Result<JSONObject> testRecord(@RequestBody String json){
        int allNum = 0;
        List<RecordEntityDto> dtos = new ArrayList<>();
        Page<RecordEntity> page = this.recordService.find(json);
        if(page.getTotalElements() >0){
            allNum = (int) page.getTotalElements();
            page.getContent().forEach(item ->{
                RecordEntityDto dto = copyEntry(item);

                UserEntry userEntry = this.userService.findByIdentifier(item.getIdentifier());
                dto.setUserName(userEntry.getUserName());
                dto.setGender(userEntry.getGender());
                dto.setProfession(userEntry.getProfession());
                dtos.add(dto);
            });
        }
        JSONObject resultJson = new JSONObject();
        resultJson.put("data",dtos);
        resultJson.put("allNum",allNum);
        return success(resultJson);
    }

    @PostMapping("/download/source")
    @ApiOperation(value = "原始数据批量下载",notes = "ids :数组，下载的结果id，为空时表示全部下载；flag：量表flag")
    public String downSourceDate(@RequestBody String json, HttpServletResponse response) throws IOException {
        //原始记录文件
        File file;
        //原始记录复制后文件
        File destFile;
        //查找原始记录路径文件夹名称
        String testDate;
        //查找原始记录路径文件夹名称
        String folderName;
        //所有记录汇总后文件夹，用来压缩
        String filePath = null;
        //原始记录文件名
        String fileName;
        //查询测试记录 不分页
        JSONObject condition = JSONObject.parseObject(json);
        List<RecordEntity> recordEntities = this.recordService.findNoPage(condition.toJSONString());
        if(!Utils.isNotBlank(recordEntities)){
            return  "无测试记录";
        }
        //加载所有的量表资源，减少数据库访问次数
        Map<String,String> subjectMap = loadAllSubject();

        for (RecordEntity recordEntity : recordEntities) {
            fileName = recordEntity.getFileName().substring(recordEntity.getFileName().lastIndexOf("/")+1);
            testDate = recordEntity.getTestDate();
            folderName = subjectMap.get(recordEntity.getFlag());
            //将日期格式 yyyy年MM月dd日HH时mm分ss秒转换成 yyyy-MM-dd
            testDate = testDate.substring(0,10).replace("年","-").replace("月","-");
            file = new File("./心理检测/检测结果/" + testDate + "/" + folderName + "/" + fileName);
            destFile = new File("./心理检测/答题记录/" + folderName + "/" + recordEntity.getIdentifier() + "-" + folderName +"-"+recordEntity.getTestDate()+ ".xlsx");
            filePath = "./心理检测/答题记录";
            //将原始文件进行复制
            FileUtils.copyFile(file, destFile);
        }

        //将复制后的整个文件夹打包
        String zipPath = filePath+  DateUtil.getNowTime(DateUtil.DATE_PARSE_PATTERNS[3]);
        ZipUtils.createZip(filePath, zipPath +".zip", false);

        File zipFile = new File(zipPath + ".zip");
        //下载压缩后文件
        if (zipFile.exists()) {
//            return downloadFile(zipFile,filePath,"答题记录-"+DateUtil.getNowTime(DateUtil.DATE_PARSE_PATTERNS[3])+".zip",response);
            return com.rf.kjb.utils.FileUtils.downloadFile(zipFile,filePath,"答题记录-"+DateUtil.getNowTime(DateUtil.DATE_PARSE_PATTERNS[3])+".zip",response);
        }
        return "下载失败";
    }

    private String downloadFile(File file ,String filePath ,String fileName,HttpServletResponse response) throws IOException {
        response.reset();
        response.setContentType("application/octet-stream");
        response.setCharacterEncoding("utf-8");
        response.setContentLength((int) file.length());
        response.setHeader("Content-Disposition", "attachment;filename=" + java.net.URLEncoder.encode(fileName, "UTF-8"));

        byte[] buffer = new byte[1024];
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        try {
            fis = new FileInputStream(file);
            bis = new BufferedInputStream(fis);
            OutputStream os = response.getOutputStream();
            int i = bis.read(buffer);
            while (i != -1) {
                os.write(buffer, 0, i);
                i = bis.read(buffer);
            }

            return "下载成功";
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            //将复制后的整个文件夹删除
            if(StringUtils.isNotBlank(filePath)){
                Path path = Paths.get(filePath);
                try (Stream<Path> walk = Files.walk(path)) {
                    walk.sorted(Comparator.reverseOrder())
                            .forEach(com.rf.kjb.utils.FileUtils::deleteDirectoryStream);
                }
                //删除生成的压缩包
                if(file.exists()) file.delete();
            }

        }
        return "下载失败";
    }

    /**
     * 加载所有的量表信息
     * @return Map<flag,name>
     */
    private Map<String, String> loadAllSubject() {
       List<SubjectEntity> subjectEntities =  this.subjectService.findAll("0");
        Map<String,String> subjectMap = new HashMap<>();
       if(Utils.isNotBlank(subjectEntities)){
           subjectEntities.forEach(item -> subjectMap.put(item.getFlag(),item.getName()));
       }
       return subjectMap;
    }

    private RecordEntityDto copyEntry(RecordEntity item) {
        RecordEntityDto dto = new RecordEntityDto();
        dto.setId(item.getId());
        dto.setCreateTime(item.getCreateTime());
        dto.setTestRecord(item.getTestRecord());
        dto.setFlag(item.getFlag());
        dto.setIdentifier(item.getIdentifier());
        dto.setFileName(item.getFileName());
        dto.setTestDate(item.getTestDate());
        dto.setImg(item.getImg());
        dto.setTestResult(item.getTestResult());
        dto.setTestResults(item.getTestResults());
        dto.setUpdateTime(item.getUpdateTime());
        dto.setName(this.subjectService.getSubjectByFlag(item.getFlag()).getName());
        return dto;
    }


}
