package com.rf.kjb.scale.rest;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.base.rest.BaseController;
import com.rf.kjb.scale.dao.model.RecordPhyEntry;
import com.rf.kjb.scale.service.RecordPhyService;
import com.rf.kjb.scale.util.DateUtil;
import com.rf.kjb.user.dao.model.UserEntry;
import com.rf.kjb.user.service.UserService;
import com.rf.kjb.utils.FileUtils;
import com.rf.kjb.utils.Result;
import com.rf.kjb.utils.Utils;
import com.rf.kjb.utils.ZipUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

/**
 * @Author:zzf
 * @Date:2022/7/28:19:20
 * @Description:
 */
@RestController
@RequestMapping("/v1/recordPhy")
@Slf4j
@Api(tags = "生理测试记录")
public class RecordPhyController extends BaseController {

    @Autowired
    private RecordPhyService recordPhyService;

    @Autowired
    private UserService userService;

    @PostMapping("/save")
    @ApiOperation(value = "保存记录",notes = "testDate:测试时间戳；testResult：测试结果；sourceData：原始数据；type：类型 1 脉搏 2心电 3脑电；fileName：文件名；identifier：用户编号")
    public Result save(@RequestBody String json){
        RecordPhyEntry recordPhyEntry = JSONObject.parseObject(json).toJavaObject(RecordPhyEntry.class);
        this.recordPhyService.save(recordPhyEntry);
        return success();
    }

    @PostMapping("/find")
    @ApiOperation(value = "查询",notes = "pageNum:当前页数 默认1  pageSize：每页记录数，默认10 startDate：测试起始时间，可为空 endDate：测试结束时间 type:类型：1为脉搏；2为心电；3为脑电，可为空 fileName:文件名称，可为空；identifier：用户编号 可为空")
    public Result<Page<RecordPhyEntry>> find(@RequestBody String json){
        JSONObject jsonObject = JSONObject.parseObject(json);
        Page<RecordPhyEntry> page = this.recordPhyService.find(jsonObject);
        return success(page);
    }

    @PostMapping("/find/{id}")
    @ApiOperation(value = "详情查询",notes = "id 主键 id")
    public Result<RecordPhyEntry> findById(@PathVariable String id ){
        RecordPhyEntry entry  = this.recordPhyService.findById(id);
        assert entry != null : "测试记录不存在";
        return success(entry);
    }

    @PostMapping("/user")
    @ApiOperation(value = "查询有生理测试记录的用户信息",notes = "pageNum:当前页数 默认1  pageSize：每页记录数，默认10 startDate：测试起始时间，可为空 endDate：测试结束时间 type:类型：1为脉搏；2为心电；3为脑电，可为空 filaName:文件名称，可为空；userName：用户名可为空")
    public Result<JSONObject> findUser(@RequestBody String json){
        List<String> identifiers;

        List<UserEntry> userEntries = new ArrayList<>();
        JSONObject jsonObject = JSONObject.parseObject(json);
        identifiers = this.recordPhyService.findIdentifiers(jsonObject);
        int count = this.recordPhyService.findIdentifiersCount(jsonObject);
        if(Utils.isNotBlank(identifiers)){
            userEntries = this.userService.findByIdentifiers(identifiers);
        }
        JSONObject resultJson = new JSONObject();
        resultJson.put("data",userEntries);
        resultJson.put("allNum",count);
        return  success(resultJson);
    }

    @PostMapping("/download")
    @ApiOperation(value = "原始数据批量下载",notes = "ids :数组，下载的结果id，为空时表示全部下载 ,flag :1为脉搏；2为心电；3为脑电")
    public String download(@RequestBody String json, HttpServletResponse response) throws IOException {
        JSONObject jsonObject = JSONObject.parseObject(json);
        List<String> ids = jsonObject.getJSONArray("ids").toJavaList(String.class);
        String  flag = jsonObject.getString("flag");
        List<RecordPhyEntry> recordPhyEntryList = this.recordPhyService.find(ids,flag);
        Set<String> fileNameSet = new HashSet<>();
        if(Utils.isNotBlank(recordPhyEntryList)){
//            HashMap<String,String> map = new HashMap<>();
////            1为脉搏；2为心电；3为脑电
//            recordPhyEntryList.forEach( item -> fileNameSet.add(item.getFileName()));
//            if(Utils.isNotBlank(fileNameSet)){
//                return downloadSourceFile(fileNameSet,flag,response);
//            }
            return downloadSourceFile(recordPhyEntryList,flag,response);
        }
        return "下载失败";
    }

    private String downloadSourceFile(List<RecordPhyEntry> recordPhyEntryList, String flag, HttpServletResponse response) throws IOException {
        //copy   file
        String path_prefix = System.getProperty("user.home");
        //目标文件目录
        String destFileParent = "./生理测试记录原始数据/";
        recordPhyEntryList.forEach(item -> {
            File file = new File(path_prefix + "/KJB/" + item.getFileName() + ".txt");
            if(file.exists()){
                //将txt文本内容转换成excel文件
                try {
                    FileUtils.txt2excel(file,destFileParent,flag,item);
                } catch (IOException e) {
                    log.error("文件下载失败：");
                    e.printStackTrace();
                    throw new RuntimeException(e);
                }
            }
        });

        //压缩文件
        ZipUtils.createZip(destFileParent,"./生理测试记录原始数据.zip",true);
        return FileUtils.downloadFile(new File("./生理测试记录原始数据.zip"),"","生理测试记录原始数据-"+ DateUtil.getCurrentDate(DateUtil.DATE_PARSE_PATTERNS[3]) +".zip",response);
    }

    private String downloadSourceFile(Set<String> fileNameSet, String flag,HttpServletResponse response) throws IOException {
        //copy   file
        String path_prefix = System.getProperty("user.home");
        //目标文件目录
        String destFileParent = "./生理测试记录原始数据/";
        fileNameSet.forEach(fileName -> {
            File file = new File(path_prefix + "/KJB/" + fileName + ".txt");
            if(file.exists()){
                //将txt文本内容转换成excel文件
                try {
                    FileUtils.txt2excel(file,destFileParent,flag);
                } catch (IOException e) {
                    log.error("文件下载失败：");
                    e.printStackTrace();
                    throw new RuntimeException(e);
                }
            }
        });
        //压缩文件
        ZipUtils.createZip(destFileParent,"./生理测试记录原始数据.zip",true);
        return FileUtils.downloadFile(new File("./生理测试记录原始数据.zip"),"","生理测试记录原始数据-"+ DateUtil.getCurrentDate(DateUtil.DATE_PARSE_PATTERNS[3]) +".zip",response);
    }

    @ApiOperation("全部导出脉搏测试指标")
    @PostMapping("/export/all/pulse/target")
    public String batchExportPulse(HttpServletResponse response) throws IOException {
        List<RecordPhyEntry> recordPhyEntryList = this.recordPhyService.findByType("1");
        if(recordPhyEntryList != null && recordPhyEntryList.size()>0){
            String fileName = "./脉搏指标-"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date())+".xlsx";
            File distFile = new File(fileName);
            Workbook wb = new XSSFWorkbook();
            //创建单元格，设置style居中，字体，单元格大小等
            CellStyle style = wb.createCellStyle();
            //设置水平对齐的样式为居中对齐;
            style.setAlignment(HorizontalAlignment.CENTER);
            //设置垂直对齐的样式为居中对齐;
            style.setVerticalAlignment(VerticalAlignment.CENTER);
            //创建表头
            String [] titles = {"用户编号","测试时间","SD2","SD1","HF_norm","SDNN","LF_norm","RMSSD","VLF","LF/HF","PNN20","PNN50","SDSD","P_total","LF","HF","0.1Hz"};

            Sheet sheet = wb.createSheet("时域指标和频域指标");
            Row titleRow = sheet.createRow(0);
            titleRow.setRowStyle(style);
            for (int i = 0; i < titles.length; i++) {
                titleRow.createCell(i).setCellValue(titles[i]);
            }
            AtomicInteger rowNum = new AtomicInteger();
            recordPhyEntryList.forEach(item ->{
                rowNum.addAndGet(1);
                Row dataRow = sheet.createRow(rowNum.get());
                JSONObject jsonObject = JSONObject.parseObject(item.getTestResult().replace("lf_nu","lf_norm").replace("hf_nu","hf_norm").toUpperCase());
                for (int i = 0; i < titles.length; i++) {
                    if(i == 0){
                        Cell dataRowCell = dataRow.createCell(i);
                        dataRowCell.setCellValue(item.getIdentifier());
                    } else if (i == 1) {
                        Cell dataRowCell = dataRow.createCell(i);
                        dataRowCell.setCellValue(item.getTestDate());
                    }else {
                        Cell dataRowCell = dataRow.createCell(i);
                        dataRowCell.setCellValue(jsonObject.getString(titles[i].toUpperCase()));
                    }
                }
            });
            OutputStream fileOut = Files.newOutputStream(distFile.toPath());
            wb.write(fileOut);
            fileOut.close();
            response.reset();
            response.setContentType("application/octet-stream");
            response.setCharacterEncoding("utf-8");
            response.setContentLength((int) distFile.length());
            response.setHeader("Content-Disposition", "attachment;filename=" + java.net.URLEncoder.encode(distFile.getName(), "UTF-8"));
            response.setHeader("filename",java.net.URLEncoder.encode(distFile.getName(), "UTF-8"));
            byte[] buffer = new byte[1024];
            FileInputStream fis = null;
            BufferedInputStream bis = null;
            try {
                fis = new FileInputStream(distFile);
                bis = new BufferedInputStream(fis);
                OutputStream os = response.getOutputStream();
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }

                return "下载成功";
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (bis != null) {
                    try {
                        bis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                if (fis != null) {
                    try {
                        fis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                distFile.delete();
            }
            return "下载成功";
        }
        return "下载失败";
    }





}
