package com.rf.kjb.scale.rest;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.rf.kjb.base.rest.BaseController;
import com.rf.kjb.excel.ExcelClass;
import com.rf.kjb.excel.ExcelUtil;
import com.rf.kjb.exception.ErrorCode;
import com.rf.kjb.scale.dao.model.*;
import com.rf.kjb.scale.service.*;
import com.rf.kjb.scale.util.DateUtil;
import com.rf.kjb.user.dao.model.UserEntry;
import com.rf.kjb.user.service.UserService;
import com.rf.kjb.utils.Constant;
import com.rf.kjb.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author zsf
 * {@code @description:接收测试结果计算结果--量表测试}
 * @date 2021/1/21 16:07
 */
@Slf4j
@RestController
@RequestMapping("/v1/result")
@Api(tags = "web端量表测试结果计算")
public class WebScaleResultController extends BaseController {

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private RecordService userRecordService;

    @Autowired
    private UserService userService;

    @Autowired
    private ScaleService scaleService;

    @Autowired
    private AnswerService answerService;

    @Autowired
    private ScaleMarksService scaleMarksService;

    @Autowired
    private DimensionService dimensionService;

    @Autowired
    private ScaleConfigService scaleConfigService;

    @Transactional(rollbackFor = Exception.class)
    @PostMapping("/{flag}")
    @ApiOperation(value = "flag:量表标志，jsonObject：测试结果")
    public Result completeTest(@RequestBody String json, @PathVariable String flag) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(json);
        UserEntry userEntity = this.userService.findByIdentifier(jsonObject.getString("identifier"));
        if(userEntity == null){
            return fail(ErrorCode.USER_NOT_EXISTS);
        }
        String userName = userEntity.getUserName();
        String userSex = userEntity.getGender() == 1 ?"男":"女";
        String testTime = DateUtil.getNowTime_CN();
        if (jsonObject.containsKey("startTime") && jsonObject.getDate("startTime") != null) {
            testTime = DateUtil.getDateStr(jsonObject.getDate("startTime"), DateUtil.CN_DATE);
        }
        String fileName = userName + Constant.SPLIT_CHAR  + userSex + Constant.SPLIT_CHAR + testTime;
        StringBuilder testResult = new StringBuilder();
        JSONArray jsonArray = jsonObject.getJSONArray("scale_result");

        // 创建返回的json对象
        JSONObject resultJson = new JSONObject(true);
        //根据flag获取全部选项
        List<ScaleEntry> scaleEntities = this.scaleService.getScaleByFlag(flag);
        //根据flag获取全部答案
        List<AnswerEntity> answerEntities = this.answerService.getAnswerByFlag(flag);
        //根据flag获取全部评分规则
        List<ScaleMarksEntity> scaleMarksEntities = this.scaleMarksService.getScaleMarksByFlag(flag);
        //根据flag获取全部维度信息
        List<DimensionEntity> dimensionEntities = this.dimensionService.getDimensionByFlag(flag);
        //根据flag获取量表名称
        SubjectEntity subjectEntity = this.subjectService.getSubjectByFlag(flag);
        resultJson.put("scaleEntities", scaleEntities);
        resultJson.put("answerEntities", answerEntities);
        resultJson.put("scaleMarksEntities", scaleMarksEntities);
        resultJson.put("dimensionEntities", dimensionEntities);
        //答题结果列表
        List<ScaleEntry> dataList = jsonArray.toJavaList(ScaleEntry.class);
        ScaleConfigEntry scaleConfigByFlag = scaleConfigService.findScaleConfigByFlag(flag);
        String className = (scaleConfigByFlag== null ?"com.rf.kjb.scale.scaleResult.NEWCOMMONScaleNEW":scaleConfigByFlag.getResultConfig());
        Class<?> cls;
        try {
            cls = Class.forName(className);
            Constructor<?> cons = cls.getConstructor(JSONArray.class, JSONObject.class);
            Object obj = cons.newInstance(jsonArray, resultJson);
            Method method = cls.getDeclaredMethod("scaleCalculate");
            resultJson = (JSONObject) method.invoke(obj);
        } catch (ClassNotFoundException | NoSuchMethodException | IllegalAccessException | InstantiationException |
                 InvocationTargetException e) {
            e.printStackTrace();
        }
        resultJson.getJSONObject("resultJson").remove("scaleEntities");
        resultJson.getJSONObject("resultJson").remove("answerEntities");
        resultJson.getJSONObject("resultJson").remove("scaleMarksEntities");
        resultJson.getJSONObject("resultJson").remove("dimensionEntities");
        if (resultJson.getJSONObject("resultJson").containsKey("resultJson") && resultJson.getJSONObject("resultJson").get("测试结果指导").equals("请认真按照实际情况如实回答")) {
            resultJson = resultJson.getJSONObject("resultJson");
            return success(resultJson, "完成");
        }
        fileName = fileName + "-" + subjectEntity.getName() + ".xlsx";
        //导出文件
        ExcelUtil.createExcelFile(ScaleEntry.class, dataList, (List<Map<String, String>>) resultJson.get("resultMapList"), new ExcelClass().contentExcel(), fileName, subjectEntity.getName());

        //计算结果
        JSONArray dbJsonList = new JSONArray();
        JSONObject dbJson = new JSONObject(true);
        Map<String, Object> newResultJson;
        if (!resultJson.containsKey("newResultJson")) {
            dbJson.put("version", 1.0);
            dbJson.put("newTableContext", "");
        } else {
            newResultJson = (Map<String, Object>) resultJson.get("newResultJson");
            dbJson.put("version", 2.0);
            dbJson.put("newTableContext", newResultJson);
        }
        resultJson = resultJson.getJSONObject("resultJson");
        // 获取JSON第一层所有的key
        Set<String> keys = resultJson.keySet();

        //列名
        List colNames = new ArrayList();
        colNames.add("项目");
        colNames.add("内容");
        dbJson.put("colNames", colNames);
        JSONArray tableContext = new JSONArray();

        // 获取第一层每个key对应的值 的类型
        for (String key : keys) {
            //列值
            List colValue = new ArrayList();
            colValue.add(resultJson.get(key));
            JSONObject row = new JSONObject();
            row.put("name", key);
            row.put("value", colValue);
            tableContext.add(row);
        }
        dbJson.put("tableContext", tableContext);
        dbJsonList.add(dbJson);
        //测试记录
        int i = 1;
        // 获取第一层每个key对应的值 的类型
        for (String key : keys) {
            if (i == keys.size()) {
                testResult.append(key).append(":").append(resultJson.get(key));
            } else {
                testResult.append(key).append(":").append(resultJson.get(key)).append(";");
            }
            i++;
        }
        String filePath = "./心理检测/检测结果/" + new SimpleDateFormat("yyyy-MM-dd").format(new Date()) + "/" + subjectEntity.getName() + "/" + fileName;
        RecordEntity userRecordEntity =  testRecord(flag, userEntity.getIdentifier(),testTime, filePath, JSON.toJSONString(dbJsonList, SerializerFeature.DisableCircularReferenceDetect),  JSON.toJSONString(dataList, SerializerFeature.DisableCircularReferenceDetect), testResult.toString());
        return success(userRecordEntity.getId(),"完成");

    }




    private RecordEntity testRecord(String flag, String identifier,  String testTime, String fileName, String testResult,  String dataList, String testResults) {
        //根据flag记录测试数
        this.subjectService.addTestNum(flag);
        //添加用户测试记录
        RecordEntity userRecordEntity = new RecordEntity();
        userRecordEntity.setFlag(flag);
        userRecordEntity.setIdentifier(identifier);
        userRecordEntity.setTestDate(testTime);
        userRecordEntity.setTestResult(testResult);
        userRecordEntity.setFileName(fileName);
        userRecordEntity.setTestRecord(dataList);
        userRecordEntity.setTestResults(testResults);
        userRecordEntity.setCreateTime(new Date());
        return this.userRecordService.save(userRecordEntity);
    }

    //@SafetyProcess
//    @ApiOperation(value = "测试结果下载")
//    @GetMapping("/download/testResult")
    public String downloadTestResult(String id, HttpServletResponse response) throws Exception {
        if (id != null) {
            RecordEntity userRecordEntity = this.userRecordService.getUserRecordById(id);
            if (userRecordEntity == null) {
                return "该记录异常，下载失败";
            }
            assert StringUtils.isNotBlank(userRecordEntity.getFileName()) : "测试记录文件名为空";
            File file = new File(userRecordEntity.getFileName());
            if (file.exists()){
                response.reset();
                response.setContentType("application/octet-stream");
                response.setCharacterEncoding("utf-8");
                response.setContentLength((int) file.length());
                response.setHeader("Content-Disposition", "attachment;filename=" + java.net.URLEncoder.encode(file.getName(), "UTF-8"));

                if (downloadFile(response, file)) return "下载成功";
            }else {
                String fileName = userRecordEntity.getFileName();
                String testDate = userRecordEntity.getTestDate();
                assert StringUtils.isNotBlank(userRecordEntity.getFlag()):"测试记录标志为空";
                String name = this.subjectService.getSubjectByFlag(userRecordEntity.getFlag()).getName();
                //将日期格式 yyyy年MM月dd日HH时mm分ss秒转换成 yyyy-MM-dd
                testDate = testDate.substring(0,10).replace("年","-").replace("月","-");
                //设置文件路径
                file = new File("./心理检测/检测结果/" + testDate + "/" + name + "/" + fileName);
                if (file.exists()) {
                    response.reset();
                    response.setContentType("application/octet-stream");
                    response.setCharacterEncoding("utf-8");
                    response.setContentLength((int) file.length());
                    response.setHeader("Content-Disposition", "attachment;filename=" + java.net.URLEncoder.encode(fileName, "UTF-8"));
                    if (downloadFile(response, file)) return "下载成功";
                }
            }


        }
        return "下载失败";
    }

    /**
     * 文件下载
     * @param response
     * @param file
     * @return
     */
    private boolean downloadFile(HttpServletResponse response, File file) {
        byte[] buffer = new byte[1024];
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        try {
            fis = new FileInputStream(file);
            bis = new BufferedInputStream(fis);
            OutputStream os = response.getOutputStream();
            int i = bis.read(buffer);
            while (i != -1) {
                os.write(buffer, 0, i);
                i = bis.read(buffer);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return false;
    }


    /**
     * 验证excel是否全部为空
     *
     * @param row      当前行
     * @param firstRow 第一行标题行
     * @return
     */
    public static boolean isAllRowEmpty(Row row, Row firstRow) {
        int count = 0;
        //单元格数量
        int rowCount = firstRow.getLastCellNum() - firstRow.getFirstCellNum();
        //判断多少个单元格为空
        for (int c = 0; c < rowCount; c++) {
            Cell cell = row.getCell(c);
            if (cell == null || cell.getCellType() == CellType.BLANK || StringUtils.isEmpty((cell + "").trim())) {
                count += 1;
            }
        }

        if (count == rowCount) {
            return true;
        }

        return false;
    }

    /**
     * 描述：对表格中数值进行格式化
     */
    private static Object getCellValue(Cell cell) {
        Object value = null;
        //格式化字符类型的数字
        DecimalFormat df = new DecimalFormat("0");
        //日期格式化
        SimpleDateFormat sdf = new SimpleDateFormat("yyy-MM-dd");
        //格式化数字
        DecimalFormat df2 = new DecimalFormat("0.00");
        switch (cell.getCellType()) {

            case STRING:
                value = cell.getRichStringCellValue().getString();
                break;
            case NUMERIC:
                if ("General".equals(cell.getCellStyle().getDataFormatString())) {
                    value = df.format(cell.getNumericCellValue());
                } else if ("m/d/yy".equals(cell.getCellStyle().getDataFormatString())) {
                    value = sdf.format(cell.getDateCellValue());
                } else {
                    value = df2.format(cell.getNumericCellValue());
                }
                break;
            case BOOLEAN:
                value = cell.getBooleanCellValue();
                break;
            case BLANK:
                value = "";
                break;
            default:
                break;
        }
        return value;
    }




    public static void zipFiles(File[] srcFiles, File zipFile) {
        // 判断压缩后的文件存在不，不存在则创建
        if (!zipFile.exists()) {
            try {
                zipFile.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        // 创建 FileOutputStream 对象
        FileOutputStream fileOutputStream = null;
        // 创建 ZipOutputStream
        ZipOutputStream zipOutputStream = null;
        // 创建 FileInputStream 对象
        FileInputStream fileInputStream = null;

        try {
            // 实例化 FileOutputStream 对象
            fileOutputStream = new FileOutputStream(zipFile);
            // 实例化 ZipOutputStream 对象
            zipOutputStream = new ZipOutputStream(fileOutputStream);
            // 创建 ZipEntry 对象
            ZipEntry zipEntry = null;
            // 遍历源文件数组
            for (int i = 0; i < srcFiles.length; i++) {
                // 将源文件数组中的当前文件读入 FileInputStream 流中
                fileInputStream = new FileInputStream(srcFiles[i]);
                // 实例化 ZipEntry 对象，源文件数组中的当前文件
                zipEntry = new ZipEntry(srcFiles[i].getName());
                zipOutputStream.putNextEntry(zipEntry);
                // 该变量记录每次真正读的字节个数
                int len;
                // 定义每次读取的字节数组
                byte[] buffer = new byte[1024];
                while ((len = fileInputStream.read(buffer)) > 0) {
                    zipOutputStream.write(buffer, 0, len);
                }
            }
            zipOutputStream.closeEntry();
            zipOutputStream.close();
            fileInputStream.close();
            fileOutputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }
    /**
     * 将Map字符串转换为Map
     *
     * @param str Map字符串
     * @return Map
     */
    public static Map<String,String> mapStringToMap(String str){
        str = str.substring(1, str.length()-1);
        String[] strs = str.split(",");
        Map<String,String> map = new HashMap<String, String>();
        for (String string : strs) {
            String key = string.split("=")[0];
            String value = string.split("=")[1];
            // 去掉头部空格
            String key1 = key.trim();
            String value1 = value.trim();
            map.put(key1, value1);
        }
        return map;
    }



}
