package com.rf.kjb.scale.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.scale.dao.model.CategoryEntry;
import com.rf.kjb.scale.dao.model.CategorySubjectEntity;
import com.rf.kjb.scale.dao.model.SubjectEntity;
import com.rf.kjb.scale.dao.repository.CategoryRepository;
import com.rf.kjb.scale.service.CategoryService;
import com.rf.kjb.utils.Utils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;


/**
 * @author zsy
 * @description:量表类别信息管理
 * @date 2021/6/17 15:55
 */
@Service
public class CategoryServiceImpl implements CategoryService {

    @Autowired
    private CategoryRepository categoryRepository;


    /**
     * 保存
     *
     * @param CategoryEntry
     */
    @Override
    public void save(CategoryEntry CategoryEntry) {
        this.categoryRepository.save(CategoryEntry);
    }



    /**
     * 名称查询
     *
     * @param categoryEntry
     * @return
     */
    @Override
    public List<CategoryEntry> getCategoryByName(CategoryEntry categoryEntry) {

        Specification<CategoryEntry> specification = (root, query, cb) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if(StringUtils.isNotBlank(categoryEntry.getEname())){
                predicateList.add(cb.equal(root.get("ename"),categoryEntry.getEname()));
            }
            if(StringUtils.isNotBlank(categoryEntry.getName())){
                predicateList.add(cb.equal(root.get("name"),categoryEntry.getName()));
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.asc(root.get("sequence")));
            return query.getRestriction();
        };
        return this.categoryRepository.findAll(specification);
    }

    /**
     * 查询所有类别
     *
     * @return
     */
    @Override
    public List<CategoryEntry> findAll() {
        return this.categoryRepository.findAll(Sort.by(Sort.Order.asc("sequence")));
    }

}
