package com.rf.kjb.scale.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.scale.dao.model.GameRecordEntity;
import com.rf.kjb.scale.dao.model.RecordPhyEntry;
import com.rf.kjb.scale.dao.repository.GameRecordRepository;
import com.rf.kjb.scale.service.GameRecordService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author:zzf
 * @Date:2023/6/7:10:26
 * @Description:
 */
@Service
public class GameRecordServiceImpl implements GameRecordService {
    @Autowired
    private GameRecordRepository gameRecordRepository;
    @Override
    public void save(GameRecordEntity gameRecordEntity) {
        this.gameRecordRepository.save(gameRecordEntity);
    }

    @Override
    public Page<GameRecordEntity> find(String json) {
        JSONObject jsonObject = JSONObject.parseObject(json);
        int pageNum = 1;
        int pageSize = 10 ;

        if(jsonObject.containsKey("pageNum")&&jsonObject.getIntValue("pageNum")>0){
            pageNum = jsonObject.getIntValue("pageNum");
        }
        if(jsonObject.containsKey("pageSize")&&jsonObject.getIntValue("pageSize")>0){
            pageSize = jsonObject.getIntValue("pageSize");
        }
        Specification<RecordPhyEntry> specification = (root, query, cb) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if(jsonObject.containsKey("id")&& StringUtils.isNotBlank(jsonObject.getString("id"))){
                predicateList.add(cb.equal(root.get("id"),jsonObject.getString("id")));
            }
            if(jsonObject.containsKey("identifier") && StringUtils.isNotBlank(jsonObject.getString("identifier"))){
                predicateList.add(cb.equal(root.get("identifier"),jsonObject.getString("identifier")));
            }
            if(jsonObject.containsKey("gameScene") && StringUtils.isNotBlank(jsonObject.getString("gameScene"))){
                predicateList.add(cb.equal(root.get("gameScene"),jsonObject.getString("gameScene")));
            }
            if(jsonObject.containsKey("equipmentType") && StringUtils.isNotBlank(jsonObject.getString("equipmentType"))){
                predicateList.add(cb.equal(root.get("equipmentType"),jsonObject.getString("equipmentType")));
            }
            if(jsonObject.containsKey("fileName") && StringUtils.isNotBlank(jsonObject.getString("fileName"))){
                predicateList.add(cb.like(root.get("fileName"),"%"+jsonObject.getString("fileName")+"%"));
            }
            if(jsonObject.containsKey("gameDifficulty")){
                predicateList.add(cb.equal(root.get("gameDifficulty"),jsonObject.getString("gameDifficulty")));
            }
            if(jsonObject.containsKey("gameTime")){
                predicateList.add(cb.equal(root.get("gameTime"),jsonObject.getString("gameTime")));
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.desc(root.get("saveTime")));
            return query.getRestriction();
        };
        return this.gameRecordRepository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }
}
