package com.rf.kjb.scale.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.scale.dao.model.RecordPhyEntry;
import com.rf.kjb.scale.dao.repository.RecordPhyRepository;
import com.rf.kjb.scale.service.RecordPhyService;
import com.rf.kjb.utils.Utils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author:zzf
 * @Date:2022/7/28:19:25
 * @Description:
 */
@Service
public class RecordPhyServiceImpl implements RecordPhyService {

    @Autowired
    private RecordPhyRepository repository;

    @Override
    public void save(RecordPhyEntry recordPhyEntry) {
        this.repository.save(recordPhyEntry);
    }

    @Override
    public Page<RecordPhyEntry> find(JSONObject jsonObject) {
        int pageNum = 1;
        int pageSize = 10 ;

        if(jsonObject.containsKey("pageNum")&&jsonObject.getIntValue("pageNum")>0){
            pageNum = jsonObject.getIntValue("pageNum");
        }
        if(jsonObject.containsKey("pageSize")&&jsonObject.getIntValue("pageSize")>0){
            pageSize = jsonObject.getIntValue("pageSize");
        }
        Specification<RecordPhyEntry> specification = (root, query, cb) -> {

            List<Predicate> predicateList = new ArrayList<>();
            if(jsonObject.containsKey("startDate")&& StringUtils.isNotBlank(jsonObject.getString("startDate"))){
                predicateList.add(cb.greaterThanOrEqualTo(root.get("testDate"),jsonObject.getString("startDate")));
            }
            if(jsonObject.containsKey("endDate") && StringUtils.isNotBlank(jsonObject.getString("endDate"))){
                predicateList.add(cb.lessThanOrEqualTo(root.get("testDate"),jsonObject.getString("endDate")+"23时59分59秒999"));
            }
            if(jsonObject.containsKey("type") && StringUtils.isNotBlank(jsonObject.getString("type"))){
                predicateList.add(cb.equal(root.get("type"),jsonObject.getString("type")));
            }
            if(jsonObject.containsKey("fileName") && StringUtils.isNotBlank(jsonObject.getString("fileName"))){
                predicateList.add(cb.like(root.get("fileName"),"%"+jsonObject.getString("fileName")+"%"));
            }
            if(jsonObject.containsKey("identifier")){
               predicateList.add(cb.equal(root.get("identifier"),jsonObject.getString("identifier")));
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.asc(root.get("testDate")));
            return query.getRestriction();
        };
        return this.repository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }

    @Override
    public List<String> findIdentifiers(JSONObject jsonObject) {
        int pageNum = 1;
        int pageSize = 10;
        if(jsonObject.containsKey("pageNum")){
            pageNum = jsonObject.getIntValue("pageNum") -1;
        }
        if(jsonObject.containsKey("pageSize")){
            pageSize = jsonObject.getIntValue("pageSize");
        }

        String startDate = jsonObject.containsKey("startDate")?jsonObject.getString("startDate"):"";
        String endDate = jsonObject.containsKey("endDate")?jsonObject.getString("endDate"):"";
        String type = jsonObject.containsKey("type")?jsonObject.getString("type"):"";
//        String userName = jsonObject.containsKey("userName")?"%"+jsonObject.getString("userName")+"%":"";
        String userName ="";
        if(jsonObject.containsKey("userName") && StringUtils.isNotBlank(jsonObject.getString("userName"))){
            userName = "%"+jsonObject.getString("userName")+"%";
        }
//        return this.repository.findIdentifiers(type,startDate,endDate,userName,pageNum-1,pageSize);
        return this.repository.findIdentifiers(type,startDate,userName,endDate,pageNum*pageSize,pageSize);
    }

    @Override
    public int findIdentifiersCount(JSONObject jsonObject) {
        String startDate = jsonObject.containsKey("startDate")?jsonObject.getString("startDate"):"";
        String endDate = jsonObject.containsKey("endDate")?jsonObject.getString("endDate"):"";
        String type = jsonObject.containsKey("type")?jsonObject.getString("type"):"";
//        String userName = jsonObject.containsKey("userName")?"%"+jsonObject.getString("userName")+"%":"";
        String userName ="";
        if(jsonObject.containsKey("userName") && StringUtils.isNotBlank(jsonObject.getString("userName"))){
            userName = "%"+jsonObject.getString("userName")+"%";
        }
        return this.repository.findIdentifiersCount(type,startDate,userName,endDate);
    }

    @Override
    public RecordPhyEntry findById(String id) {
        return this.repository.findById(id).orElse(null);
    }

    @Override
    public List<RecordPhyEntry> find(List<String> recordingIds, String flag) {
        Specification<RecordPhyEntry> specification = (root, query, cb) -> {

            List<Predicate> predicateList = new ArrayList<>();
           if(Utils.isNotBlank(recordingIds)){
               predicateList.add(root.get("id").in(recordingIds));
           }
           if(StringUtils.isNotBlank(flag)){
                predicateList.add(cb.equal(root.get("type"),flag));
           }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.desc(root.get("testDate")));
            return query.getRestriction();
        };
        return this.repository.findAll(specification);
    }

    @Override
    public List<RecordPhyEntry> findAll(JSONObject jsonObject) {
        return null;
    }

    @Override
    public List<RecordPhyEntry> findByType(String type) {
        return this.repository.findByType(type);
    }

    @Override
    public void deleteByIdentifier(String identifier) {
        this.repository.deleteByIdentifier(identifier);
    }
}
