package com.rf.kjb.scale.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.scale.dao.model.RecordEntity;
import com.rf.kjb.scale.dao.model.SubjectEntity;
import com.rf.kjb.scale.dao.repository.RecordRepository;
import com.rf.kjb.scale.dao.repository.SubjectRepository;
import com.rf.kjb.scale.service.RecordService;
import com.rf.kjb.user.dao.repository.UserRepository;
import com.rf.kjb.utils.Utils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

/**
 * @author zsy
 * @description:用户测试记录
 * @date 2021/6/17 15:55
 */
@Slf4j
@Service
public class RecordServiceImpl implements RecordService {

    @Autowired
    private RecordRepository userRecordRepository;

    @Autowired
    private SubjectRepository subjectRepository;

    @Autowired
    private UserRepository userRepository;

    /**
     * 保存
     *
     * @param userRecordEntity
     */
    @Override
    public RecordEntity save(RecordEntity userRecordEntity) {
        return this.userRecordRepository.save(userRecordEntity);
    }

    /**
     * 根据id查询数据
     *
     * @return
     */
    @Override
    public RecordEntity getUserRecordById(String id) {
        return this.userRecordRepository.getUserRecordById(id);
    }


    /**
     * 查询全量数据
     *
     * @return
     */
    @Override
    public List<RecordEntity> findAll() {
        return this.userRecordRepository.findAll();
    }


    /**
     * @param json
     * @return
     */
    @Override
    public List<String> findIdentifiers(String json) {
        JSONObject jsonObject = JSONObject.parseObject(json);
        int pageNum = 1;
        int pageSize = 10;
        if(jsonObject.containsKey("pageNum")){
            pageNum = jsonObject.getIntValue("pageNum") ;
        }
        if(jsonObject.containsKey("pageSize")){
            pageSize = jsonObject.getIntValue("pageSize");
        }

        String startDate = jsonObject.containsKey("startDate")?jsonObject.getString("startDate"):"";
        String endDate = jsonObject.containsKey("endDate")?jsonObject.getString("endDate"):"";
        String userName ="";
        String flag = "";
        if(jsonObject.containsKey("flag") && StringUtils.isNotBlank(jsonObject.getString("flag"))){
            flag = jsonObject.getString("flag");
        }
        if(jsonObject.containsKey("userName") && StringUtils.isNotBlank(jsonObject.getString("userName"))){
            userName = "%"+jsonObject.getString("userName")+"%";
        }
        return this.userRecordRepository.findIdentifiers(flag,startDate,userName,endDate,(pageNum-1)*pageSize,pageSize);


    }

    @Override
    public int findIdentifiersCount(String json) {
        JSONObject jsonObject = JSONObject.parseObject(json);
        String startDate = jsonObject.containsKey("startDate")?jsonObject.getString("startDate"):"";
        String endDate = jsonObject.containsKey("endDate")?jsonObject.getString("endDate"):"";
        String userName ="";
        String flag = "";
        if(jsonObject.containsKey("flag") && StringUtils.isNotBlank(jsonObject.getString("flag"))){
            flag = jsonObject.getString("flag");
        }
        if(jsonObject.containsKey("userName") && StringUtils.isNotBlank(jsonObject.getString("userName"))){
            userName = "%"+jsonObject.getString("userName")+"%";
        }
        return this.userRecordRepository.findIdentifiersCount(flag,startDate,userName,endDate);
    }

    @Override
    public Page<RecordEntity> find(String json) {
        int pageNum = 1;
        int pageSize = 10 ;
        JSONObject jsonObject = JSONObject.parseObject(json);
        if(jsonObject.containsKey("pageNum")&&jsonObject.getIntValue("pageNum")>0){
            pageNum = jsonObject.getIntValue("pageNum");
        }
        if(jsonObject.containsKey("pageSize")&&jsonObject.getIntValue("pageSize")>0){
            pageSize = jsonObject.getIntValue("pageSize");
        }

        Specification<RecordEntity> specification = createSpecification(jsonObject);
        return this.userRecordRepository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }

    @Override
    public RecordEntity findById(String id) {
        return this.userRecordRepository.findById(id).orElse(null);
    }

    @Override
    public List<RecordEntity> findNoPage(String json) {
        JSONObject jsonObject = JSONObject.parseObject(json);
        Specification<RecordEntity> specification = createSpecification(jsonObject);

        return this.userRecordRepository.findAll(specification);
    }

    @Override
    public void deleteByIdentifier(String identifier) {
        this.userRecordRepository.deleteByIdentifier( identifier);
    }

    private Specification<RecordEntity> createSpecification(JSONObject jsonObject) {
        return (root, query, cb) -> {

            List<Predicate> predicateList = new ArrayList<>();
            if(jsonObject.containsKey("startDate")&& StringUtils.isNotBlank(jsonObject.getString("startDate"))){
                predicateList.add(cb.greaterThanOrEqualTo(root.get("testDate"),jsonObject.getString("startDate")));
            }
            if(jsonObject.containsKey("endDate") && StringUtils.isNotBlank(jsonObject.getString("endDate"))){
                predicateList.add(cb.lessThanOrEqualTo(root.get("testDate"),jsonObject.getString("endDate")+"23时59分59秒"));
            }
            if(jsonObject.containsKey("identifier")){
                predicateList.add(cb.equal(root.get("identifier"),jsonObject.getString("identifier")));
            }
            if(jsonObject.containsKey("id") && StringUtils.isNotBlank(jsonObject.getString("id"))){
                predicateList.add(cb.equal(root.get("id"),jsonObject.getString("id")));
            }
            if(jsonObject.containsKey("ids") && jsonObject.getJSONArray("ids") != null && jsonObject.getJSONArray("ids").size()>0){
                List<String> ids = jsonObject.getJSONArray("ids").toJavaList(String.class);
                predicateList.add(root.<String>get("id").in(ids));
            }
            if(jsonObject.containsKey("scaleName") && StringUtils.isNotBlank(jsonObject.getString("scaleName"))){
                List<SubjectEntity> list = this.subjectRepository.findByName("%"+jsonObject.getString("scaleName")+"%");
                if(Utils.isNotBlank(list)){
                    List<String> flags = new ArrayList<>();
                    list.forEach(item -> flags.add(item.getFlag()));
                    predicateList.add(root.get("flag").in(flags));
                }else{
                    //以量表名称为筛选条件但未找到时，将查询条件修改为flag = ""返回 空数据
                    predicateList.add(cb.equal(root.get("flag"),""));
                }
            }
            if(jsonObject.containsKey("flag") && StringUtils.isNotBlank(jsonObject.getString("flag"))){
                predicateList.add(cb.equal(root.get("flag"),jsonObject.getString("flag")));
            }
            if(jsonObject.containsKey("userName") && StringUtils.isNotBlank(jsonObject.getString("userName"))){

                List<String> identifiers = this.userRepository.findIdentifierByName("%"+jsonObject.getString("userName")+"%");
                predicateList.add(root.get("identifier").in(identifiers));
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.desc(root.get("testDate")));
            return query.getRestriction();
        };
    }
}
