package com.rf.kjb.scale.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.scale.dao.model.CategorySubjectEntity;
import com.rf.kjb.scale.dao.model.SubjectEntity;
import com.rf.kjb.scale.dao.repository.SubjectRepository;
import com.rf.kjb.scale.service.CategorySubjectService;
import com.rf.kjb.scale.service.SubjectService;
import com.rf.kjb.utils.Constant;
import com.rf.kjb.utils.Utils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author zzf
 * @description:
 * @date 2021/5/17 9:44
 */
@Service
public class SubjectServiceImpl implements SubjectService {
    @Autowired
    private SubjectRepository subjectRepository;

    @Autowired
    private CategorySubjectService categorySubjectService;

    /**
     * 获取量表和认知任务列表
     *
     * @return
     */
    @Override
    public List<SubjectEntity> findAll(int pageNum, int pageSize) {
        int beginIndex = (pageNum - 1) * pageSize;
        return this.subjectRepository.findAll(beginIndex, pageSize);
    }

    /**
     * 检索，根据中文名或录入人
     *
     * @param pageNum
     * @param pageSize
     * @param searchName
     * @return
     */
    @Override
    public List<SubjectEntity> findByNameOrUploader(int pageNum, int pageSize, String searchName) {
        int beginIndex = (pageNum - 1) * pageSize;
        return this.subjectRepository.findByNameOrUploader(beginIndex, pageSize, searchName);
    }

    /**
     * 保存
     *
     * @param subjectEntity
     */
    @Override
    public void save(SubjectEntity subjectEntity) {
        //subjectEntity.encrypt();
        this.subjectRepository.save(subjectEntity);
    }


    /**
     * 查询
     *
     * @param type 0：量表  1：认知任务
     * @return
     */
    @Override
    public List<SubjectEntity> findAll(String type) {
        return this.subjectRepository.findAll(type);
    }








    /**
     * 根据flag查询
     *
     * @return
     */
    @Override
    public SubjectEntity getSubjectByFlag(String flag) {
        return this.subjectRepository.getSubjectByFlag(flag);
    }

    /**
     * 根据flag记录测试数
     *
     * @return
     */
    @Override
    public void addTestNum(String flag) {
        this.subjectRepository.addTestNum(flag);
    }

    /**
     * 修改量表测试结果效验状态
     *
     * @return
     */
    @Override
    public void updateScaleStatus(String flag, String status) {
        this.subjectRepository.updateScaleStatus(flag, status);
    }

    @Override
    public void update(SubjectEntity subjectEntity) {
        this.subjectRepository.updateSubject(subjectEntity.getDescription(), subjectEntity.getId());
    }

    @Override
    public Page<SubjectEntity> find(String json) {
        int pageNum = 1;
        int pageSize = 10 ;
        JSONObject jsonObject = JSONObject.parseObject(json);
        if(jsonObject.containsKey("pageNum")&&jsonObject.getIntValue("pageNum")>0){
            pageNum = jsonObject.getIntValue("pageNum");
        }
        if(jsonObject.containsKey("pageSize")&&jsonObject.getIntValue("pageSize")>0){
            pageSize = jsonObject.getIntValue("pageSize");
        }
        Specification<SubjectEntity> specification = (root, query, cb) -> {

            List<Predicate> predicateList = new ArrayList<>();
            if(jsonObject.containsKey("flag")&&StringUtils.isNotBlank(jsonObject.getString("flag"))){
                predicateList.add(cb.equal(root.get("flag"),jsonObject.getString("flag")));
            }
            if(jsonObject.containsKey("name")&&StringUtils.isNotBlank(jsonObject.getString("name"))){
                predicateList.add(cb.like(root.get("name"),"%"+jsonObject.getString("name")+"%"));
            }
            //分类英文名
            if(jsonObject.containsKey("typeName") && StringUtils.isNotBlank(jsonObject.getString("typeName"))){
                List<CategorySubjectEntity> list = this.categorySubjectService.findByEname(jsonObject.getString("typeName"));
                if(Utils.isNotBlank(list)){
                    List<String> flagList = new ArrayList<>();
                    list.forEach(categorySubjectEntity -> flagList.add(categorySubjectEntity.getFlag()));
                    predicateList.add(root.<String>get("flag").in(flagList));
                }
            }
            //排除交互式量表
            if(jsonObject.containsKey("role")&&jsonObject.getIntValue("role")==0){
                predicateList.add(cb.notEqual(root.get("flag"), Constant.SPECIAL_SCALE_FLAG_SRSS));
                predicateList.add(cb.notEqual(root.get("flag"), Constant.SPECIAL_SCALE_FLAG_DASS_21));
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.asc(root.get("no")));
            return query.getRestriction();
        };
        return this.subjectRepository.findAll(specification, PageRequest.of(pageNum-1,pageSize));

    }


}
