package com.rf.kjb.user.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.user.dao.model.UserEntry;
import com.rf.kjb.user.dao.repository.UserRepository;
import com.rf.kjb.user.service.UserService;
import com.rf.kjb.utils.Utils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

/**
 * {@code @Author:zzf}
 * {@code @Date:2022/7/22:18:17}
 * {@code @Description:}
 */
@Service
public class UserServiceImpl implements UserService {
    @Autowired
    private UserRepository repository;
    /**
     * identifier ---> user
     *
     * @param identifier
     * @return
     */
    @Override
    public UserEntry findByIdentifier(String identifier) {
        return this.repository.findByIdentifier(identifier);
    }

    /**
     * save
     *
     * @param userEntry
     */
    @Override
    public void save(UserEntry userEntry) {
        this.repository.save(userEntry);
    }

    /**
     * reset password
     *
     * @param identifier
     * @param password
     */
    @Override
    public void resetPasswordByIdentifier(String identifier, String password) {
        this.repository.updatePasswordByIdentifier(identifier,password);
    }

    /**
     * 分页查询
     *
     * @param json
     * @return
     */
    @Override
    public Page<UserEntry> find(String json) {

        int pageNum = 1;
        int pageSize = 20 ;
        JSONObject jsonObject = JSONObject.parseObject(json);
        if (jsonObject.containsKey("pageNum")&&jsonObject.getIntValue("pageNum") > 0)
            pageNum = jsonObject.getIntValue("pageNum");
        if (jsonObject.containsKey("pageSize")&&jsonObject.getIntValue("pageSize") > 0)
            pageSize = jsonObject.getIntValue("pageSize");
        Specification<UserEntry> specification = (root, query, cb) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if((jsonObject.containsKey("userName")&& StringUtils.isNotBlank(jsonObject.getString("userName")))){
                predicateList.add(cb.like(root.get("userName"), "%"+jsonObject.getString("userName").trim()+"%"));
            }
            if(jsonObject.containsKey("role")){
                predicateList.add(cb.equal(root.get("role"),jsonObject.getIntValue("role")));
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.desc(root.get("createTime")),cb.desc(root.get("updateTime")));

            return query.getRestriction();
        };
        return  this.repository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }

    /**
     * 用户名查询identifier
     *
     * @param userName
     * @return
     */
    @Override
    public List<String> findIdentifierByName(String userName) {
        return  this.repository.findIdentifierByName(userName);

    }

    @Override
    public List<UserEntry> findByIdentifiers(List<String> identifiers) {
        Specification<UserEntry> specification = (root, query, cb) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if(Utils.isNotBlank(identifiers)){
                predicateList.add(root.<String>get("identifier").in(identifiers));
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            return query.getRestriction();
        };
        return  this.repository.findAll(specification);
    }

    @Override
    public UserEntry findByIdentifierAndPassword(String identifier, String password) {

        return this.repository.findByIdentifierAndPassword( identifier,  password);
    }

    @Override
    public void deleteByIdentifier(String identifier) {
        this.repository.deleteByIdentifier( identifier);
    }
}
